package it.unimi.dsi.util;

/*		 
 * DSI utilities
 *
 * Copyright (C) 2010-2013 Sebastiano Vigna 
 *
 *  This library is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU Lesser General Public License as published by the Free
 *  Software Foundation; either version 3 of the License, or (at your option)
 *  any later version.
 *
 *  This library is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License
 *  for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program; if not, see <http://www.gnu.org/licenses/>.
 * 
 */

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;
import it.unimi.dsi.fastutil.longs.LongOpenHashSet;

import java.util.Random;

import org.junit.Ignore;
import org.junit.Test;

import com.google.common.hash.Funnels;

public class BloomFilterTest {
	
	@Test
	public void testAdd() {
		BloomFilter<Void> bloomFilter = new BloomFilter<Void>( 10, 20, BloomFilter.VOID_FUNNEL ); // High precision 
		assertTrue( bloomFilter.add( "test" ) );
		assertFalse( bloomFilter.add( "test" ) );
		assertTrue( bloomFilter.add( "foo" ) );
		assertTrue( bloomFilter.add( "bar" ) );
		assertEquals( 3, bloomFilter.size64() );

		assertTrue( bloomFilter.contains( "test" ) );
		assertTrue( bloomFilter.contains( "foo" ) );
		assertTrue( bloomFilter.contains( "bar" ) );

		assertFalse( bloomFilter.contains( "42" ) );
		assertFalse( bloomFilter.contains( "42" ) );

		bloomFilter.clear();
		assertTrue( bloomFilter.add( new byte[] { 0, 1 } ) );
		assertFalse( bloomFilter.add( new byte[] { 0, 1 } ) );
		assertTrue( bloomFilter.add( new byte[] { 1, 2 } ) );
		assertTrue( bloomFilter.add( new byte[] { 1, 0 } ) );
		assertEquals( 3, bloomFilter.size64() );
	}

	@Test
	public void testConflictsStrings() {
		BloomFilter<Void> bloomFilter = new BloomFilter<Void>( 1000, 8, BloomFilter.VOID_FUNNEL ); // Low precision
		LongOpenHashSet longs = new LongOpenHashSet();
		Random random = new XorShiftStarRandom( 5 );
		
		for( int i = 1000; i-- != 0; ) {
			final long l = random.nextLong();
			longs.add( l );
			final String s = Long.toBinaryString( l );
			bloomFilter.add( s );
			assertTrue( bloomFilter.contains( s ) );
		}
		
		assertEquals( longs.size(), bloomFilter.size64() );
	}

	@Test
	public void testConflictsLongs() {
		BloomFilter<Long> bloomFilter = new BloomFilter<Long>( 1000, 8, Funnels.longFunnel() ); // Low precision
		LongOpenHashSet longs = new LongOpenHashSet();
		Random random = new XorShiftStarRandom( 5 );
		
		for( int i = 1000; i-- != 0; ) {
			final long l = random.nextLong();
			longs.add( l );
			final Long o = Long.valueOf( l );
			bloomFilter.add( o );
			assertTrue( bloomFilter.contains( o ) );
		}
		
		assertEquals( longs.size(), bloomFilter.size64() );
	}

	@Test
	public void testNegativeSeed() {
		BloomFilter<Long> bloomFilter = new BloomFilter<Long>( 1000, 8, Funnels.longFunnel() );
		assertTrue( bloomFilter.add( "test" ) );
		assertFalse( bloomFilter.add( "test" ) );
	}

	@Ignore
	@Test
	public void testConflictsBig() {
		BloomFilter<Long> bloomFilter = new BloomFilter<Long>( 1000000000, 30, Funnels.longFunnel() ); // Low precision
		LongOpenHashSet longs = new LongOpenHashSet();
		Random random = new XorShiftStarRandom( 5 );
		
		for( int i = 10000000; i-- != 0; ) {
			final long l = random.nextLong();
			longs.add( l );
			final Long o = Long.valueOf( l );
			bloomFilter.add( o );
				}
		
		assertEquals( longs.size(), bloomFilter.size64() );
	}

	@Test
	public void testZeroFunctions() {
		BloomFilter<Void> bloomFilter = new BloomFilter<Void>( 10, 0, BloomFilter.VOID_FUNNEL ); 
		assertFalse( bloomFilter.add( "test" ) );
		assertEquals( 0, bloomFilter.size64() );
	}
}
